//-----------------------------------------------------------------------------
// VSMSGW.CPP
//
// Encapsulates sending/receiving of VSMS messages to RTCU Units thru 
// the Logic IO RTCU Communication Hub
// The module uses the gwprot.dll which contains the low-level protocol stack used
// for communication to the RTCU Communication Hub.
//
//-----------------------------------------------------------------------------
#include "stdafx.h"
#include "VSMSGW.h"
#include "gwprot\gwprot.h" // Include support for low-level RTCU Communication Hub protocol
#include <process.h>


#pragma pack(1)
  struct rmonReqGPRSSendData {
    short         cmd;
    short         datalength;
    unsigned char data[GW_MAXDATASIZE_EXT];
  };
#pragma pack()


//-----------------------------------------------------------------------------
// Holds the parameters supplied in the vsmsgwInit() function, that are needed later on...
//-----------------------------------------------------------------------------
cbfuncText    cbText=NULL;
cbfuncPDU     cbPDU=NULL;
cbfuncPackage cbPackage=NULL;
void          *cbArgText=NULL;
void          *cbArgPDU=NULL;
void          *cbArgPackage=NULL;


//-----------------------------------------------------------------------------
// Entry point for the DLL
// Currently not used for anything....
//-----------------------------------------------------------------------------
BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved) {
  switch (ul_reason_for_call)	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
  }
  return TRUE;
}


//-----------------------------------------------------------------------------
// Thread that listens for requests.
// The thread will block in gwReceiveRequest() until an request is received.
// If the request is an VSMS message, it will call the correct callback function (Text or PDU)
// which will give the return code back, which should be sent back in the gwSendResponse() call
//-----------------------------------------------------------------------------
static void thgwReceiveRequest(void *arg) {
  unsigned long src_node;                 // source node-id
  unsigned long TNO;                      // TNO
  unsigned short cmd;                     // command code.
  unsigned char data[GW_MAXDATASIZE_EXT]; // data returned
  unsigned short data_len;                // length of data

  while (true) {
    int rc=gwReceiveRequest(0,         // channel
                            &src_node, // source node-id
                            &TNO,      // TNO
                            &cmd,      // command code.
                            data,      // data returned
                            &data_len  // length of data
                           );

    if (rc==GWRC_OK) {
      switch (cmd) {
        //received a VSMS message:
        case 0x0017:
          // If it is a Text SMS we received, send it to the registered callback handler        
          if (data[0]==0) {
            gwSendResponse(src_node, 
                           TNO, 
                           // Call the callbackfunction registered, and use the return value as the answer back to the RTCU
                           // If no cb function registered, return 2 as error.
                           cbText ? cbText(src_node, (char*)&data[2], cbArgText) : 2, 
                           NULL, 
                           0);
          // If it is a PDU SMS we received, send it to the registered callback handler        
          } else if (data[0]==1) {
            gwSendResponse(src_node, 
                           TNO, 
                           // Call the callbackfunction registered, and use the return value as the answer back to the RTCU
                           // If no cb function registered, return 2 as error.
                           cbPDU ? cbPDU(src_node, &data[2], data[1], cbArgPDU) : 2, 
                           NULL, 
                           0);
          }
          break;

        //received a GPRS data package:
        case 0x0044:
          {
            rmonReqGPRSSendData *sd = (rmonReqGPRSSendData*)data;
            gwSendResponse(src_node, 
                           TNO, 
                           // Call the callbackfunction registered, and use the return value as the answer back to the RTCU
                           // If no cb function registered, return 2 as error.
                           cbPackage ? cbPackage(src_node, sd->data, sd->datalength, cbArgPackage) : 2, 
                           NULL, 
                           0);
          }
          break;
      }
    } else {
      Sleep(100);
    }
  }
}


//-----------------------------------------------------------------------------
// Initialize the connection to the RTCU Communication Hub
// Input:
//   MyNodeID  : The nodeid for the PC application. If set to 0, it will be assigned by the RTCU Communication Hub
//   GWIP     : The IP address (or symbolic name) of the RTCU Communication Hub
//   GWPort   : The pportnumber the RTCU Communication Hub listens on
//   GWKey    : The key value (an 8 character password) used to access the RTCU Communication Hub. Must be null terminated,
//   SMSText   : A callbackfunction that will be called whenever an Text SMS is received
//   SMSPDU    : A callbackfunction that will be called whenever an PDU SMS is received
//   arg       : A user supplied 32 bit variable that will be supplied to the callback functions when they get called
// Returns 0 if OK, else error (Look at GWRC_xx codes)
//-----------------------------------------------------------------------------
VSMSGW_API int _cdecl vsmsgwInit(int MyNodeID, const char *GWIP, int GWPort, const char *GWKey,  cbfuncText SMSText, cbfuncPDU SMSPDU, void *arg) {
  int rc;

  // Enable LPS
  gwEnableExtended(GW_MAXDATASIZE_EXT);

  // First open connection to the RTCU Communication Hub, this call will return immediatly, connection continues in background,
  // use gwIsConnected() to determine when the connection is established.
  rc=gwOpen(MyNodeID, GWIP, GWPort, GWKey);

  // Remember the address of the two callback functions supplied, and the user "argument"
  cbText=SMSText;
  cbPDU=SMSPDU;
  cbArgText=arg;
  cbArgPDU=arg;

  // Start the thread that will listen for incoming transactions
  _beginthread(thgwReceiveRequest, 0, NULL);

  return rc;
}


//-----------------------------------------------------------------------------
// Returns TRUE if connected to the RTCU Communication Hub, FALSE if not connected
//-----------------------------------------------------------------------------
VSMSGW_API bool _cdecl vsmsgwIsConnected(void) {
  return gwIsConnected()==GWRC_OK ? true:false;
}


//-----------------------------------------------------------------------------
// This function will return this nodes nodeid
// This function is used especially when a dynamic node-id is requested (by 
// setting MyNodeID to 0 in the vsmsgwInit() function
//-----------------------------------------------------------------------------
VSMSGW_API int _cdecl vsmsgwGetMyNodeID(unsigned long* MyNodeID) {
  return gwGetMyNodeID(MyNodeID);
}


//-----------------------------------------------------------------------------
// Send a Text SMS message to the specified NodeID, the returncode from the
// RTCU unit, will be contained in rc.
// Input:
//   HisNodeID : The nodenumber (serialnumber) of the receiving RTCU unit
//   str       : The string to send to the receiving RTCU unit. Maximum size is 160 characters + NULL terminator.
// Output:
//   rc        : The return code from the receiving RTCU unit, 0 if OK, else error
// Returns 0 if OK, else error (Look at GWRC_xx codes)
//-----------------------------------------------------------------------------
VSMSGW_API int _cdecl vsmsgwSendSMS(unsigned int HisNodeID, const char *str, int *rc) {
  
  unsigned char txdata[238];
  unsigned char rxdata[238];
  unsigned short datarx_len, response;
  // We have to build a buffer for the transaction, as the type and length goes before the actual data...
  txdata[0]=0; // VSMS type, 0=text, 1=PDU
  txdata[1]=0; // Only used if PDU
  // Copy the supplied text string into the buffer
  strcpy((char*)&txdata[2], str); // The text is copied to the transmit buffer

  int rrc= gwTransaction(0,                  // Channel
                        //IN:
                         HisNodeID,          // destination node-id
                         0x0017,             // command code.
                         txdata,             // data to send.
                         2+strlen(str)+1,    // length of data to send
                        //OUT:
                         &response,          // return code
                         rxdata,             // data received
                         &datarx_len         // length of data recieved
                        );
  *rc=response; // This is the return code from the RTCU unit to the actual VSMS message
  return rrc;
}


//-----------------------------------------------------------------------------
// Send a PDU SMS message to the specified NodeID, the returncode from the
// RTCU unit, will be contained in rc
// Input:
//   HisNodeID : The nodenumber (serialnumber) of the receiving RTCU unit
//   data      : The data to send to the receiving RTCU unitMaximum size is 140 bytes.
//   length    : The length of data to send
// Output:
//   rc        : The return code from the receiving RTCU unit, 0 if OK, else error
// Returns 0 if OK, else error (Look at GWRC_xx codes)
//-----------------------------------------------------------------------------
VSMSGW_API int _cdecl vsmsgwSendPDU(unsigned int HisNodeID, unsigned char *data, int length, int *rc) {
  unsigned char txdata[238];
  unsigned char rxdata[238];
  unsigned short datarx_len, response;

  // We have to build a buffer for the transaction, as the type and length goes before the actual data...
  txdata[0]=1;      // VSMS type, 0=text, 1=PDU
  txdata[1]=length; // Only used if PDU
  // Copy the supplied text string into the buffer
  memcpy((char*)&txdata[2], data, length);   // The text

  int rrc= gwTransaction(0,                  // Channel
                        //IN:
                         HisNodeID,          // destination node-id
                         0x0017,             // command code.
                         txdata,             // data to send.
                         2+length,           // length of data to send
                        //OUT:
                         &response,          // return code
                         rxdata,             // data received
                         &datarx_len         // length of data recieved
                        );
  *rc=response; // This is the return code from the RTCU unit to the actual VSMS message
  return rrc;
}


//-----------------------------------------------------------------------------
// Send a binary packet to the specified NodeID, the returncode from the
// RTCU unit, will be contained in rc
// Input:
//   HisNodeID : The nodenumber (serialnumber) of the receiving RTCU unit
//   data      : The data to send to the receiving RTCU unit. Maximum size is 4064 bytes.
//   length    : The length of data to send
// Output:
//   rc        : The return code from the receiving RTCU unit, 0 if OK, else error
// Returns 0 if OK, else error (Look at GWRC_xx codes)
//-----------------------------------------------------------------------------
VSMSGW_API int _cdecl vsmsgwSendDataPackage(unsigned int HisNodeID, unsigned char *data, int length, int *rc) {
  // Var
  struct rmonReqGPRSSendData sd;
  unsigned short             response;

  // Build package
  sd.cmd        = 0x0044;
  sd.datalength = length;
  memcpy(sd.data, data, length);

  // Send package
  int rrc= gwTransaction(0,                                       // Channel
                        //IN:
                         HisNodeID,                               // destination node-id
                         0x0044,                                  // command code.
                         (void*)&sd,                              // data to send.
                         sizeof(sd) - sizeof(sd.data) + length,   // length of data to send
                        //OUT:
                         &response,                               // return code
                         NULL,                                    // data received
                         NULL                                     // length of data recieved
                        );

  // Completed
  *rc = response; // This is the return code from the RTCU unit to the actual VSMS message
  return rrc;
}


//-----------------------------------------------------------------------------
// Initialize the connection to the RTCU Communication Hub
// Input:
//   MyNodeID  : The nodeid for the PC application. If set to 0, it will be assigned by the RTCU Communication Hub
//   GWIP     : The IP address (or symbolic name) of the RTCU Communication Hub
//   GWPort   : The pportnumber the RTCU Communication Hub listens on
//   GWKey    : The key value (an 8 character password) used to access the RTCU Communication Hub. Must be null terminated,
//   CryptKey  : The encryption key used to access the RTCU Communication Hub.
// Returns 0 if OK, else error (Look at GWRC_xx codes)
//-----------------------------------------------------------------------------
VSMSGW_API int _cdecl vsmsgwInitAdv(
                                    int           MyNodeID,
                                    const char    *GWIP,
                                    int           GWPort,
                                    const char    *GWKey,
                                    unsigned char CryptKey[16]
                                   ) {
  // Var
  int rc;

  // First open connection to the RTCU Communication Hub, this call will return immediatly, connection continues in background,
  // use gwIsConnected() to determine when the connection is established.
  rc = gwSetEncryptionKey(CryptKey);
  if (rc) return rc;

  // Enable LPS
  gwEnableExtended(GW_MAXDATASIZE_EXT);

  rc = gwOpen(MyNodeID, GWIP, GWPort, GWKey);
  if (rc) return rc;

  // Start the thread that will listen for incoming transactions
  _beginthread(thgwReceiveRequest, 0, NULL);

  // Completed
  return GWRC_OK;
}


//-----------------------------------------------------------------------------
// Send callbackfunction that will be called whenever an Text SMS is received
// Input:
//   SMSText   : A callbackfunction that will be called whenever an Text SMS
//               is received.
//   arg       : A user supplied 32 bit variable that will be supplied to the
//               callback function when it gets called.
//-----------------------------------------------------------------------------
VSMSGW_API void _cdecl vsmsgwSetSMSTextCB(cbfuncText SMSText, void *arg) {
  // Remember the address of the two callback functions supplied, and the user "argument"
  cbText    = SMSText;
  cbArgText = arg;
}


//-----------------------------------------------------------------------------
// Send callbackfunction that will be called whenever a PDU SMS is received
// Input:
//   SMSPDU    : A callbackfunction that will be called whenever a PDU SMS is
//               received.
//   arg       : A user supplied 32 bit variable that will be supplied to the
//               callback function when it gets called.
//-----------------------------------------------------------------------------
VSMSGW_API void _cdecl vsmsgwSetSMSPDUCB(cbfuncPDU SMSPDU, void *arg) {
  // Remember the address of the two callback functions supplied, and the user "argument"
  cbPDU    = SMSPDU;
  cbArgPDU = arg;
}


//-----------------------------------------------------------------------------
// Send callbackfunction that will be called whenever a data package is received
// Input:
//   PACKAGE   : A callbackfunction that will be called whenever a data package
//               is received.
//   arg       : A user supplied 32 bit variable that will be supplied to the
//               callback function when it gets called.
//-----------------------------------------------------------------------------
VSMSGW_API void _cdecl vsmsgwSetDataPackageCB(cbfuncPackage PACKAGE, void *arg) {
  // Remember the address of the two callback functions supplied, and the user "argument"
  cbPackage    = PACKAGE;
  cbArgPackage = arg;
}

