//-----------------------------------------------------------------------------
// VSMSGW.H
//
// Encapsulates sending/receiving of VSMS messages to RTCU Units through
// the Logic IO RTCU Communication Hub
//
//-----------------------------------------------------------------------------

#ifdef VSMSGW_EXPORTS
#define VSMSGW_API __declspec(dllexport)
#else
#define VSMSGW_API __declspec(dllimport)
#endif


// Different returncodes from the functions:
#define GWRC_OK				    	0     /* operation successfull */
#define GWRC_ERROR				  1     /* unspecified error */
#define GWRC_NOT_CON				2     /* not connected */
#define GWRC_TIMEOUT				3     /* a timeout occured */
#define GWRC_INV_LEN				5     /* invalid length */
#define GWRC_IS_OPEN        7     /* is already open */
#define GWRC_NOT_OPEN       8     /* is not open */
#define GWRC_INV_PARM       9     /* invalid parameters */
#define GWRC_DST_UNREACH    10     /* destination node is unreachable */



//-----------------------------------------------------------------------------
// Definition of callback function for receiving Text SMS messages
// Input :
//   HisNodeID  : The nodeid of the sender (the serialnumber of the RTCU that sent the message)
//   str        : The text string sent by the RTCU.Max size is 160 characters + a null-terminator
//   arg        : A user supplied 32 bit variable that was set when vsmsgwInit() was called
// Output:
//   rc         : This is the return code that should be returned to the RTCU Unit, 0 if OK, else error  
//-----------------------------------------------------------------------------
typedef int(_cdecl *cbfuncText)(int HisNodeID, char *str, void *arg);


//-----------------------------------------------------------------------------
// Definition of callback function for receiving PDU SMS messages
// Input :
//   HisNodeID  : The nodeid of the sender (the serialnumber of the RTCU that sent the message)
//   data       : The data sent by the RTCU.Max size is 140 bytes.
//   length     : The length of data sent by the RTCU
//   arg        : A user supplied 32 bit variable that was set when vsmsgwInit() was called
// Output:
//   rc         : This is the return code that should be returned to the RTCU Unit, 0 if OK, else error  
//-----------------------------------------------------------------------------
typedef int(_cdecl *cbfuncPDU)(int HisNodeID, unsigned char *data, int length, void *arg);


//-----------------------------------------------------------------------------
// Definition of callback function for receiving packets
// Input :
//   HisNodeID  : The nodeid of the sender (the serialnumber of the RTCU that sent the message)
//   data       : The data sent by the RTCU. Max size is 4064 bytes.
//   length     : The length of data sent by the RTCU
//   arg        : A user supplied 32 bit variable that was set when vsmsgwInit() was called
// Output:
//   rc         : This is the return code that should be returned to the RTCU Unit, 0 if OK, else error  
//-----------------------------------------------------------------------------
typedef int(_cdecl *cbfuncPackage)(int HisNodeID, unsigned char *data, int length, void *arg);


//-----------------------------------------------------------------------------
// Initialize the connection to the RTCU Communication Hub
// Input:
//   MyNodeID  : The nodeid for the PC application. If set to 0, it will be assigned by the RTCU Communication Hub
//   GWIP     : The IP address (or symbolic name) of the RTCU Communication Hub
//   GWPort   : The pportnumber the RTCU Communication Hub listens on
//   GWKey    : The key value (an 8 character password) used to access the RTCU Communication Hub. Must be null terminated,
//   SMSText   : A callbackfunction that will be called whenever an Text SMS is received
//   SMSPDU    : A callbackfunction that will be called whenever a PDU SMS is received
//   arg       : A user supplied 32 bit variable that will be supplied to the callback functions when they get called
// Returns 0 if OK, else error (Look at GWRC_xx codes)
//-----------------------------------------------------------------------------
VSMSGW_API int _cdecl vsmsgwInit(int MyNodeID, const char *GWIP, int GWPort, const char *GWKey, cbfuncText SMSText, cbfuncPDU SMSPDU, void *arg);


//-----------------------------------------------------------------------------
// Returns true if connected to the RTCU Communication Hub
//-----------------------------------------------------------------------------
VSMSGW_API bool _cdecl vsmsgwIsConnected(void);


//-----------------------------------------------------------------------------
// This function will return this nodes nodeid
// This function is used especially when a dynamic node-id is requested (by 
// setting MyNodeID to 0 in the vsmsgwInit() function
//-----------------------------------------------------------------------------
VSMSGW_API int _cdecl vsmsgwGetMyNodeID(unsigned long* MyNodeID);


//-----------------------------------------------------------------------------
// Send a Text SMS message to the specified NodeID, the returncode from the
// RTCU unit, will be contained in rc.
// Input:
//   HisNodeID : The nodenumber (serialnumber) of the receiving RTCU unit
//   str       : The string to send to the receiving RTCU unit. Maximum size is 160 characters + NULL terminator.
// Output:
//   rc        : The return code from the receiving RTCU unit, 0 if OK, else error
// Returns 0 if OK, else error (Look at GWRC_xx codes)
//-----------------------------------------------------------------------------
VSMSGW_API int _cdecl vsmsgwSendSMS(unsigned int HisNodeID, const char *str,  int *rc);


//-----------------------------------------------------------------------------
// Send a PDU SMS message to the specified NodeID, the returncode from the
// RTCU unit, will be contained in rc
// Input:
//   HisNodeID : The nodenumber (serialnumber) of the receiving RTCU unit
//   data      : The data to send to the receiving RTCU unitMaximum size is 140 bytes.
//   length    : The length of data to send
// Output:
//   rc        : The return code from the receiving RTCU unit, 0 if OK, else error
// Returns 0 if OK, else error (Look at GWRC_xx codes)
//-----------------------------------------------------------------------------
VSMSGW_API int _cdecl vsmsgwSendPDU(unsigned int HisNodeID, unsigned char *data, int length, int *rc);


//-----------------------------------------------------------------------------
// Send a data package to the specified NodeID, the returncode from the
// RTCU unit, will be contained in rc
// Input:
//   HisNodeID : The nodenumber (serialnumber) of the receiving RTCU unit
//   data      : The data to send to the receiving RTCU unit. Maximum size is 4064 bytes.
//   length    : The length of data to send
// Output:
//   rc        : The return code from the receiving RTCU unit, 0 if OK, else error
// Returns 0 if OK, else error (Look at GWRC_xx codes)
//-----------------------------------------------------------------------------
VSMSGW_API int _cdecl vsmsgwSendDataPackage(unsigned int HisNodeID, unsigned char *data, int length, int *rc);


//-----------------------------------------------------------------------------
// Initialize the connection to the RTCU Communication Hub
// Input:
//   MyNodeID  : The nodeid for the PC application. If set to 0, it will be assigned by the RTCU Communication Hub
//   GWIP     : The IP address (or symbolic name) of the RTCU Communication Hub
//   GWPort   : The pportnumber the RTCU Communication Hub listens on
//   GWKey    : The key value (an 8 character password) used to access the RTCU Communication Hub. Must be null terminated,
//   CryptKey  : The encryption key used to access the RTCU Communication Hub.
// Returns 0 if OK, else error (Look at GWRC_xx codes)
//-----------------------------------------------------------------------------
VSMSGW_API int _cdecl vsmsgwInitAdv(int MyNodeID, const char *GWIP, int GWPort, const char *GWKey, unsigned char CryptKey[16]);


//-----------------------------------------------------------------------------
// Set callback function that will be called whenever an Text SMS is received
// Input:
//   SMSText   : A callbackfunction that will be called whenever an Text SMS
//               is received.
//   arg       : A user supplied 32 bit variable that will be supplied to the
//               callback function when it gets called.
//-----------------------------------------------------------------------------
VSMSGW_API void _cdecl vsmsgwSetSMSTextCB(cbfuncText SMSText, void *arg);


//-----------------------------------------------------------------------------
// Set callback function that will be called whenever a PDU SMS is received
// Input:
//   SMSPDU    : A callbackfunction that will be called whenever a PDU SMS is
//               received.
//   arg       : A user supplied 32 bit variable that will be supplied to the
//               callback function when it gets called.
//-----------------------------------------------------------------------------
VSMSGW_API void _cdecl vsmsgwSetSMSPDUCB(cbfuncPDU SMSPDU, void *arg);


//-----------------------------------------------------------------------------
// Set callback function that will be called whenever a data package is received
// Input:
//   PACKAGE   : A callbackfunction that will be called whenever a data package
//               is received.
//   arg       : A user supplied 32 bit variable that will be supplied to the
//               callback function when it gets called.
//-----------------------------------------------------------------------------
VSMSGW_API void _cdecl vsmsgwSetDataPackageCB(cbfuncPackage PACKAGE, void *arg);

