/***************************** C HEADER FILE *********************************
**
**  Project:    Logic IO RTCU Gateway Protocol
**  Filename:   GWPROT.H 
**  
******************************************************************************
**
**  VERSION HISTORY:
**  ----------------
**  Version:    1.00
**  Date:       2002
**  Revised by: Kim Erik Thomsen
**  Description:
**      - Original version.
**
**  Version:    2.00
**  Date:       December 12, 2006
**  Revised by: Michael Olsen
**  Description:
**      - gwSetEncryptionKey function added for support of data encryption.
**
**  Version:    2.01
**  Date:       August 20, 2007
**  Revised by: Michael Olsen
**  Description:
**      - Header cleanup.
**      - gwGetVer function added.
**
**  Version:    3.00
**  Date:       June 18, 2015
**  Revised by: Michael Olsen
**  Description:
**      - Large Packet Support is implemented.
**      - gwmEnableExtended and gwEnableExtended functions added.
**      - gwmGetExtended and gwGetExtended functions added.
**
*****************************************************************************/
#ifndef _GWPROT__H_
#define _GWPROT__H_

#ifdef __cplusplus
  extern "C" {
#endif


/*****************************************************************************
**
**    MODULES USED 
**
*****************************************************************************/


/*****************************************************************************
**
**    DEFINITIONS AND MACROS
**
*****************************************************************************/
#define GWPROTCC             __cdecl  

#define GW_MAXDATASIZE_STD       512     /* maximum size of data send/received in one packet */
#define GW_MAXDATASIZE_EXT      4096     /* maximum size of data send/received in one packet, extended frames */
#define GW_MAXDATASIZE          GW_MAXDATASIZE_EXT

#define GWRC_OK                    0     /* operation successful */
#define GWRC_ERROR                 1     /* unspecified error */
#define GWRC_NOT_CON               2     /* not connected */
#define GWRC_TIMEOUT               3     /* a timeout occurred */
#define GWRC_INV_LEN               5     /* invalid length */
#define GWRC_IS_OPEN               7     /* is already open */
#define GWRC_NOT_OPEN              8     /* is not open */
#define GWRC_INV_PARM              9     /* invalid parameters */
#define GWRC_DST_UNREACH          10     /* destination node is unreachable */
#define GWRC_NOT_INIT             11     /* module has not been initialized */
#define GWRC_TOO_LARGE            12     /* the destination does not support large packets or packet too large */
#define GWRC_TOO_MANY             13     /* the destination have too many pending messages */


/****************************************************************************
**
**    TYPEDEFS AND STRUCTURES
**
****************************************************************************/
struct hSession;


/****************************************************************************
** 
**    EXPORTED VARIABLES
**
****************************************************************************/


/****************************************************************************
**
**    EXPORTED FUNCTIONS
**
****************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// Query whether the gateway communication module has been initialized
//
// Input:
//      None.
//
// Output:
//      None.
//
// Returns:
//     GWRC_OK       - Gateway communication module has been initialized
//     GWRC_NOT_INIT - Gateway communication module has NOT been initialized 
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwIsInitialized(void);

/////////////////////////////////////////////////////////////////////////////
// Retrieve the version number of the GWPROT library.
//
// Input:
//      None.
//
// Output:
//      None.
//
// Returns:
//     The version scaled by 100.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwGetVer(void);

/////////////////////////////////////////////////////////////////////////////
// Create a new gateway communication session, this function will allocate
// and initialize a new session which can be used with the gwm* function.
//
// When complete with the session it must be freed with gwmDelete.
//
// Output:
//      pSession        - the session handle.
//
// Returns:
//      GWRC_OK          - Success.
//      GWRC_ERROR       - General error.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwmCreate(struct hSession **pSession);

/////////////////////////////////////////////////////////////////////////////
// This will open a connection to a Gateway.
// Must be called before any other function in this DLL (except
// gwSetParameters and gwSetEncryptionKey).
//
// Input:
//      pSession     - Pointer to the session.
//      node_id      - Our NodeId. Interval: 0..999999. (0 = Dynamic NodeId).
//      ip_addr      - String containing Gateway IP address. (can be symbolic)
//      port         - The Gateway IP port.
//      key          - String containing Gateway key. (Max. 8 characters)
//
// Output:
//      None.
//
// Returns:
//     GWRC_OK       - Success.
//     GWRC_IS_OPEN  - Gateway communication is already open.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwmOpen(
                                struct hSession *pSession,
                                unsigned long   node_id,
                                const char      *ip_addr,
                                unsigned short  port,
                                const char      key[9]
                              );

/////////////////////////////////////////////////////////////////////////////
// Query whether the gateway communication is open.
//
// Input:
//      pSession     - Pointer to the session.
//
// Output:
//      None.
//
// Returns:
//     GWRC_OK       - Gateway communication is open.
//     GWRC_NOT_OPEN - Gateway communication is not open.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwmIsOpened(struct hSession *pSession);

/////////////////////////////////////////////////////////////////////////////
// Sets the TLS parameters of a session.
//
// Input:
//      pSession                   - Pointer to the session.
//      server                     - The path to the PEM file used to verify
//                                   the server certificate.
//      cert                       - The client certificate to present to
//                                   the server.
//      key                        - The private key of the client.
//      password                   - The password of the private key.
//
// Output:
//      None.
//
// Returns:
//     GWRC_OK        - Success.
//     GWRC_ERROR     - Invalid input parameter
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwmSetTLSParameters(
                                            struct hSession* pSession,
                                            char*            server,
                                            char*            cert,
                                            char*            key,
                                            char*            password
                                           );

/////////////////////////////////////////////////////////////////////////////
// Sets the Advanced connection parameters of a session.
//
// Input:
//      pSession                   - Pointer to the session.
//      gw_max_connection_attempt  - max number of connection attempts before GPRS re-connects.
//                                   interval: 1..60. (default: 3) (NOT USED ON PC IMPLEMENTATION)
//      gw_max_send_req_attempt    - max number of send-request attempt before send fails.
//                                   interval: 1..60 (default: 3).
//      gw_response_timeout        - time waiting for response in seconds.
//                                   interval: 5..60 (def: 30 seconds).
//      gw_alive_freq              - frequency for sending self-transactions in seconds
//                                   interval: 0..60000 (def: 300 seconds). 
//                                   0 (zero) will disable self-trans.
//
// Output:
//      None.
//
// Returns:
//     GWRC_OK        - Success.
//     GWRC_ERROR     - Invalid input parameter
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwmSetParameters(
                                         struct hSession *pSession,
                                         unsigned char   gw_max_connection_attempt,
                                         unsigned char   gw_max_send_req_attempt,     
                                         unsigned short  gw_response_timeout,
                                         unsigned short  gw_alive_freq
                                        );

/////////////////////////////////////////////////////////////////////////////
// Changes the key used to Encrypt the data send to, and Decrypt the data
// received from, the Gateway.
// If the key only contains 0 (zero)'s encryption is disabled.
//
// Input:
//      pSession      - Pointer to the session.
//      key           - The 128 bit long encryption key.
//
// Output:
//      None.
//
// Returns:
//     GWRC_OK        - Success.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwmSetEncryptionKey(
                                            struct hSession *pSession,
                                            unsigned char   key[16]
                                           );

/////////////////////////////////////////////////////////////////////////////
// Enable/disable compression of data send to/from the gateway.
//
// Input:
//      pSession      - Pointer to the session.
//      enable        - 0 = disable compression, <> 0 = Enable compression.
//
// Output:
//      None.
//
// Returns:
//     GWRC_OK        - Success.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwmEnableCompression(
                                             struct hSession *pSession,
                                             unsigned char   enable
                                            );

/////////////////////////////////////////////////////////////////////////////
// Enable/disable the use of extended frames, and set maximum allowed
// frame size.
//
// Input:
//      pSession      - Pointer to the session.
//      size          - 0          = disable extended frames
//                      512 - 4096 = Max. allowed size of extended frames.
//
// Output:
//      None.
//
// Returns:
//     GWRC_OK        - Success.
//     GWRC_ERROR     - Invalid session
//     GWRC_INV_LEN   - Invalid size.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwmEnableExtended(
                                          struct hSession *pSession,
                                          unsigned short  size
                                         );

/////////////////////////////////////////////////////////////////////////////
// Query the maximum frame size allowed by the gateway.
//
// Input:
//      pSession      - Pointer to the session.
//
// Output:
//      None.
//
// Returns:
//     The maximum allowed frame size. (512 - 4096)
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwmGetExtended(struct hSession *pSession);

/////////////////////////////////////////////////////////////////////////////
// Close the gateway communication.
//
// Input:
//      pSession     - Pointer to the session.
//
// Output:
//      None.
//
// Returns:
//     GWRC_OK       - Gateway communication is closed.
//     GWRC_NOT_OPEN - Gateway communication is not open.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwmClose(struct hSession *pSession);

/////////////////////////////////////////////////////////////////////////////
// Query this nodes node_id.
// This function is used especially when a dynamic node-id is requested.
//
// Input:
//      pSession      - Pointer to the session.
//
// Output:
//      node_id       - Our NodeId.
//
// Returns:
//      GWRC_OK       - Success.
//      GWRC_NOT_OPEN - Gateway communication is not open.
//      GWRC_NOT_CON  - Not connected to a Gateway.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwmGetMyNodeID(
                                       struct hSession *pSession,
                                       unsigned long   *node_id
                                      );

/////////////////////////////////////////////////////////////////////////////
// Query whether a connection to the gateway is present.
//
// Input:
//      pSession     - Pointer to the session.
//
// Output:
//      None.
//
// Returns:
//      GWRC_OK      - Connected to a Gateway.
//      GWRC_NOT_CON - Not connected to a Gateway.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwmIsConnected(struct hSession *pSession);

/////////////////////////////////////////////////////////////////////////////
// Makes a transaction (request/response) with the specific dst_node via
// the gateway.
//
// Input:
//      pSession         - Pointer to the session.
//      channel          - The channel used for the transaction
//                         interval: 0..7.
//      dst_node         - The NodeID of the destination.
//      cmd              - The command code of the request.
//      datatx           - Pointer to a buffer that contains the data to send.
//      datatx_len       - The length of the data to send. 
//                         interval: 0..GW_MAXDATASIZE.
//      datarx           - Pointer to a buffer where the received date is stored.
//
// Output:
//      rc               - The return code from the destenation.
//      datarx_len       - The length of the received data. (Max. GW_MAXDATASIZE)
//
// Returns:
//      GWRC_OK          - Success.
//      GWRC_NOT_OPEN    - Connection is not opened.
//      GWRC_NOT_CON     - Not connected to a Gateway.
//      GWRC_INV_LEN     - Invalid data length.
//      GWRC_INV_PARM    - Invalid channel.
//      GWRC_DST_UNREACH - The destination was not connected to Gateway
//      GWRC_TOO_LARGE   - Data length is too large for the destination
//      GWRC_TOO_MANY    - There are too many pending messages for the destination
//      GWRC_ERROR       - General error.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwmTransaction(
                                       struct hSession *pSession,
                                       unsigned char   channel,
                                       unsigned long   dst_node,
                                       unsigned short  cmd,
                                       const void      *datatx,
                                       unsigned short  datatx_len,
                                       unsigned short  *rc,
                                       void            *datarx,
                                       unsigned short  *datarx_len
                                      );

/////////////////////////////////////////////////////////////////////////////
// Receive a request from a foreign node via the gateway.
// Will block until a request is available.
//
// Input:
//      pSession         - Pointer to the session.
//      channel          - The channel to where the request is coming.
//                         interval: 0..7.
//      data             - Pointer to a buffer where the received date is stored.
//
// Output:
//      src_node         - The NodeID for the source of the request.
//      TNO              - The TNO of the request.
//      cmd              - The command requested by source
//      data_len         - The length of the received data. (Max. GW_MAXDATASIZE)
//
// Returns:
//      GWRC_OK          - Success.
//      GWRC_NOT_OPEN    - Connection is not opened.
//      GWRC_NOT_CON     - Not connected to a Gateway.
//      GWRC_ERROR       - General error.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwmReceiveRequest(
                                          struct hSession *pSession,
                                          unsigned char   channel,
                                          unsigned long   *src_node,
                                          unsigned long   *TNO,
                                          unsigned short  *cmd,
                                          void            *data,
                                          unsigned short  *data_len
                                         );

/////////////////////////////////////////////////////////////////////////////
// Receive a request from a foreign node via the gateway.
// Will block until a request is available or timeout expires.
//
// Input:
//      pSession         - Pointer to the session.
//      channel          - The channel to where the request is comming.
//                         interval: 0..7.
//      data             - Pointer to a buffer where the received date is stored.
//
// Output:
//      src_node         - The NodeID for the source of the request.
//      TNO              - The TNO of the request.
//      cmd              - The command requested by source
//      data_len         - The length of the received data. (Max. GW_MAXDATASIZE)
//
// Returns:
//      GWRC_OK          - Success.
//      GWRC_TIMEOUT     - Timeout.
//      GWRC_NOT_OPEN    - Connection is not opened.
//      GWRC_NOT_CON     - Not connected to a Gateway.
//      GWRC_ERROR       - General error.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwmReceiveRequestT(
                                           struct hSession *pSession,
                                           unsigned char   channel,
                                           unsigned long   *src_node,
                                           unsigned long   *TNO,
                                           unsigned short  *cmd,
                                           void            *data,
                                           unsigned short  *data_len,
                                           unsigned int    timeout
                                          );

/////////////////////////////////////////////////////////////////////////////
// Send a response back to a node that initiated the transaction.
//
// Input:
//      pSession         - Pointer to the session.
//      dst_node         - The NodeID of the destination.
//      TNO              - The TNO of the received request / transaction.
//      rc               - The Return code of the received request / transaction.
//      data             - Pointer to a buffer that contains the data to send.
//      data_len         - The length of the data to send. 
//
// Output:
//      None.
//
// Returns:
//      GWRC_OK          - Success.
//      GWRC_NOT_OPEN    - Connection is not opened.
//      GWRC_NOT_CON     - Not connected to a Gateway.
//      GWRC_ERROR       - General error.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwmSendResponse(
                                        struct hSession *pSession,
                                        unsigned long   dst_node,
                                        unsigned long   TNO,
                                        unsigned short  rc,
                                        const void      *data,
                                        unsigned short  data_len
                                       );

/////////////////////////////////////////////////////////////////////////////
// Delete a gateway communication session, this function will free the the
// allocated data, making the pointer invalid.
//
// Output:
//      pSession        - the session handle, will be cleared if it was
//                        possible delete the hole session.
// Returns:
//      GWRC_OK          - Success.
//      GWRC_ERROR       - General error.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwmDelete(struct hSession **pSession);

/////////////////////////////////////////////////////////////////////////////
// This will initialize/open the gateway communication.
// Must be called before any other function in this DLL (except
// gwSetParameters and gwSetEncryptionKey).
//
// Input:
//      node_id   - Our NodeId. Interval: 0..999999. (0 = Dynamic NodeId).
//      ip_addr   - String containing Gateway IP address. (can be symbolic)
//      port      - The Gateway IP port.
//      key       - String containing Gateway key. (Max. 8 characters)
//
// Output:
//      None.
//
// Returns:
//     GWRC_OK       - Success.
//     GWRC_IS_OPEN  - Gateway communication is already open.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwOpen(
                               unsigned long  node_id,
                               const char     *ip_addr,
                               unsigned short port,
                               const char     key[9]
                              );

/////////////////////////////////////////////////////////////////////////////
// Query whether the gateway communication is open.
//
// Input:
//      None.
//
// Output:
//      None.
//
// Returns:
//     GWRC_OK       - Gateway communication is open.
//     GWRC_NOT_OPEN - Gateway communication is not open.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwIsOpened(void);

/////////////////////////////////////////////////////////////////////////////
// Sets the TLS parameters of a session.
//
// Input:
//      server                     - The path to the PEM file used to verify
//                                   the server certificate.
//      cert                       - The client certificate to present to
//                                   the server.
//      key                        - The private key of the client.
//      password                   - The password of the private key.
//
// Output:
//      None.
//
// Returns:
//     GWRC_OK        - Success.
//     GWRC_ERROR     - Invalid input parameter
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwSetTLSParameters(
                                           char*            server,
                                           char*            cert,
                                           char*            key,
                                           char*            password
                                          );

/////////////////////////////////////////////////////////////////////////////
// Sets the Advanced parameters.
//
// Input:
//      gw_max_connection_attempt  - max number of connection attempts before GPRS re-connects.
//                                   interval: 1..60. (default: 3) (NOT USED ON PC IMPLEMENTATION)
//      gw_max_send_req_attempt    - max number of send-request attempt before send fails.
//                                   interval: 1..60 (default: 3).
//      gw_response_timeout        - time waiting for response in seconds.
//                                   interval: 5..60 (def: 30 seconds).
//      gw_alive_freq              - frequency for sending self-transactions in seconds
//                                   interval: 0..60000 (def: 300 seconds). 
//                                   0 (zero) will disable self-trans.
//
// Output:
//      None.
//
// Returns:
//     GWRC_OK        - Success.
//     GWRC_ERROR     - Invalid input parameter
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwSetParameters(
                                        unsigned char  gw_max_connection_attempt,
                                        unsigned char  gw_max_send_req_attempt,     
                                        unsigned short gw_response_timeout,
                                        unsigned short gw_alive_freq
                                       );

/////////////////////////////////////////////////////////////////////////////
// Changes the key used to Encrypt the data send to, and Decrypt the data
// received from, the Gateway.
// If the key only contains 0 (zero)'s encryption is disabled.
//
// Input:
//      key           - The 128 bit long encryption key.
//
// Output:
//      None.
//
// Returns:
//     GWRC_OK        - Success.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwSetEncryptionKey(unsigned char key[16]);

/////////////////////////////////////////////////////////////////////////////
// Enable/disable the use of extended frames, and set maximum allowed
// frame size.
//
// Input:
//      size          - 0          = disable extended frames
//                      512 - 4096 = Max. allowed size of extended frames.
//
// Output:
//      None.
//
// Returns:
//     GWRC_OK        - Success.
//     GWRC_INV_LEN   - Invalid size.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwEnableExtended(unsigned short size);

/////////////////////////////////////////////////////////////////////////////
// Query the maximum frame size allowed by the gateway.
//
// Input:
//      None.
//
// Output:
//      None.
//
// Returns:
//     The maximum allowed frame size. (512 - 4096)
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwGetExtended();

/////////////////////////////////////////////////////////////////////////////
// Close the gateway communication.
//
// Input:
//      None.
//
// Output:
//      None.
//
// Returns:
//     GWRC_OK       - Gateway communication is closed.
//     GWRC_NOT_OPEN - Gateway communication is not open.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwClose(void);

/////////////////////////////////////////////////////////////////////////////
// Query this nodes node_id.
// This function is used especially when a dynamic node-id is requested.
//
// Input:
//      None.
//
// Output:
//      node_id       - Our NodeId.
//
// Returns:
//      GWRC_OK       - Success.
//      GWRC_NOT_OPEN - Gateway communication is not open.
//      GWRC_NOT_CON  - Not connected to a Gateway.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwGetMyNodeID(unsigned long* node_id);

/////////////////////////////////////////////////////////////////////////////
// Query whether a connection to the gateway is present.
//
// Input:
//      None.
//
// Output:
//      None.
//
// Returns:
//      GWRC_OK      - Connected to a Gateway.
//      GWRC_NOT_CON - Not connected to a Gateway.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwIsConnected(void);

/////////////////////////////////////////////////////////////////////////////
// Makes a transaction (request/response) with the specific dst_node via
// the gateway.
//
// Input:
//      channel          - The channel used for the transaction
//                         interval: 0..7.
//      dst_node         - The NodeID of the destination.
//      cmd              - The command code of the request.
//      datatx           - Pointer to a buffer that contains the data to send.
//      datatx_len       - The length of the data to send. 
//                         interval: 0..GW_MAXDATASIZE.
//      datarx           - Pointer to a buffer where the received date is stored.
//
// Output:
//      rc               - The return code from the destenation.
//      datarx_len       - The length of the received data. (Max. GW_MAXDATASIZE)
//
// Returns:
//      GWRC_OK          - Success.
//      GWRC_NOT_OPEN    - Connection is not opened.
//      GWRC_NOT_CON     - Not connected to a Gateway.
//      GWRC_INV_LEN     - Invalid data length.
//      GWRC_INV_PARM    - Invalid channel.
//      GWRC_DST_UNREACH - The destination was not connected to Gateway
//      GWRC_TOO_LARGE   - Data length is too large for the destination
//      GWRC_TOO_MANY    - There are too many pending messages for the destination
//      GWRC_ERROR       - General error.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwTransaction(
                                      unsigned char  channel,
                                      unsigned long  dst_node,
                                      unsigned short cmd,
                                      const void     *datatx,
                                      unsigned short datatx_len,
                                      unsigned short *rc,
                                      void           *datarx,
                                      unsigned short *datarx_len
                                     );

/////////////////////////////////////////////////////////////////////////////
// Receive a request from a foreign node via the gateway.
// Will block until a request is available.
//
// Input:
//      channel          - The channel to where the request is coming.
//                         interval: 0..7.
//      data             - Pointer to a buffer where the received date is stored.
//
// Output:
//      src_node         - The NodeID for the source of the request.
//      TNO              - The TNO of the request.
//      cmd              - The command requested by source
//      data_len         - The length of the received data. (Max. GW_MAXDATASIZE)
//
// Returns:
//      GWRC_OK          - Success.
//      GWRC_NOT_OPEN    - Connection is not opened.
//      GWRC_NOT_CON     - Not connected to a Gateway.
//      GWRC_ERROR       - General error.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwReceiveRequest(
                                         unsigned char  channel,
                                         unsigned long  *src_node,
                                         unsigned long  *TNO,
                                         unsigned short *cmd,
                                         void           *data,
                                         unsigned short *data_len
                                        );

/////////////////////////////////////////////////////////////////////////////
// Receive a request from a foreign node via the gateway.
// Will block until a request is available or timeout expires.
//
// Input:
//      channel          - The channel to where the request is comming.
//                         interval: 0..7.
//      data             - Pointer to a buffer where the received date is stored.
//
// Output:
//      src_node         - The NodeID for the source of the request.
//      TNO              - The TNO of the request.
//      cmd              - The command requested by source
//      data_len         - The length of the received data. (Max. GW_MAXDATASIZE)
//
// Returns:
//      GWRC_OK          - Success.
//      GWRC_TIMEOUT     - Timeout.
//      GWRC_NOT_OPEN    - Connection is not opened.
//      GWRC_NOT_CON     - Not connected to a Gateway.
//      GWRC_ERROR       - General error.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwReceiveRequestT(
                                          unsigned char  channel,
                                          unsigned long  *src_node,
                                          unsigned long  *TNO,
                                          unsigned short *cmd,
                                          void           *data,
                                          unsigned short *data_len,
                                          unsigned int   timeout
                                         );

/////////////////////////////////////////////////////////////////////////////
// Send a response back to a node that initiated the transaction.
//
// Input:
//      dst_node         - The NodeID of the destination.
//      TNO              - The TNO of the received request / transaction.
//      rc               - The Return code of the received request / transaction.
//      data             - Pointer to a buffer that contains the data to send.
//      data_len         - The length of the data to send. 
//
// Output:
//      None.
//
// Returns:
//      GWRC_OK          - Success.
//      GWRC_NOT_OPEN    - Connection is not opened.
//      GWRC_NOT_CON     - Not connected to a Gateway.
//      GWRC_ERROR       - General error.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short GWPROTCC gwSendResponse(
                                       unsigned long  dst_node,
                                       unsigned long  TNO,
                                       unsigned short rc,
                                       const void     *data,
                                       unsigned short data_len
                                      );


#ifdef __cplusplus
  }
#endif


#endif  /* _GWPROT__H_ */
/****************************************************************************
**
**    EOF 
**
****************************************************************************/
